/***************************************************************************
 * Copyright 2001-2003 The Exoplatform        All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.backup.impl;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

/**
 * Author : Nhu Dinh Thuan
 *          nhudinhthuan@yahoo.com
 * Dec 20, 2005
 */
public class ZipFolder  {
  
  public void addToArchive( File input, File output) throws Exception {
    output = createFolder( output, false);
    String path = input.getAbsolutePath();    
    ZipOutputStream out = 
      new ZipOutputStream( new BufferedOutputStream( new FileOutputStream( output)));    
    final int BUFFER = 2048;
    byte data[] = new byte[BUFFER];    
    BufferedInputStream origin = null;
    
    List<File> files = getFiles( input);  
    if( files == null || files.size() < 1) return;
    for (Iterator<File> iter = files.iterator(); iter.hasNext();) {
      File f = iter.next();      
      String filePath = f.getAbsolutePath();     
      if (filePath.startsWith( path)) filePath = filePath.substring( path.length());     
      FileInputStream fi = new FileInputStream(f);       
      origin = new BufferedInputStream(fi, BUFFER);      
      ZipEntry entry = new ZipEntry(filePath);
      out.putNextEntry(entry);       
      int count;
      while ((count = origin.read(data, 0, BUFFER)) != -1)
        out.write(data, 0, count);
      origin.close();      
    }
    out.close();    
  }
  
  public void extractToFiles( File input, String output) throws Exception { 
    BufferedOutputStream dest = null;   
    FileInputStream fis = new  FileInputStream( input);
    ZipInputStream zis = new  ZipInputStream(new BufferedInputStream(fis));
    ZipEntry entry;       
    while((entry = zis.getNextEntry()) != null) {             
      int count;
      final int BUFFER = 2048;
      byte data[] = new byte[BUFFER];     
      FileOutputStream fos = null;
      if( output != null)
         fos = new FileOutputStream( createFolder(new File(output+entry.getName()), false));
      else        
        fos = new FileOutputStream( createFolder(new File(entry.getName()), false));         
      dest = new BufferedOutputStream(fos, BUFFER);
      while ((count = zis.read(data, 0, BUFFER)) != -1)  dest.write(data, 0, count);     
      dest.flush();
      dest.close();
    }
    zis.close();    
  }
  
  private File createFolder( File file, boolean folder) throws Exception {   
    if( file.getParentFile() != null)  createFolder( file.getParentFile(), true);
    if( file.exists()) return file;    
    if( file.isDirectory() || folder) file.mkdir();
    else  file.createNewFile();
    return file;
  }
  
  private List<File> getFiles(File dir) {
    final List <File> files = new LinkedList<File>();    
    dir.listFiles(new FileFilter() {
      public boolean accept(File f) {
        if (f.isDirectory()) return false;          
        files.add(f);
        return true;
      }
    });      
    dir.listFiles(new FileFilter() {
      public boolean accept(File f) {
        if (f.isFile()) return false;        
        files.addAll( getFiles(f));
        return false;
      }
    });
    return files;
  } 
}