/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.chart.impl;

import java.awt.GradientPaint;
import java.awt.image.BufferedImage;
import java.util.Iterator;

import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.chart.Chart;
import org.exoplatform.services.chart.ChartService;
import org.exoplatform.services.chart.Data;
import org.exoplatform.services.chart.ExoCategoryItemLabelGenerator;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;
/**
 * Created by The eXo Platform SARL
 * Author : Hung Nguyen
 *          nqhungvn@users.sourceforge.net
 * Jul 18, 2005
 */
public class ChartServiceImpl implements ChartService {
  private ExoCache cache_ ;
  
  public ChartServiceImpl(CacheService cservice) throws Exception {
    cache_ = cservice.getCacheInstance("ChartService:BufferedImage") ;
  }
  
  public BufferedImage createChart(Chart chart) throws Exception {
    if(chart==null || chart.getChartData()==null || chart.getChartData().isEmpty()) return null;    
    if(Chart.PIE_CHART == chart.getChartType()) {
      return createPieChart(chart);
    }
    if(Chart.COLUMN_CHART == chart.getChartType()) {
      return createBarChart(chart);
    }
    return null;
  }
  
  public String getChartID(Chart chart) throws Exception {
    if (chart==null || chart.getChartData()==null ||chart.getChartData().isEmpty()) return null;
    BufferedImage image = createChart(chart) ;
    String id = Integer.toString(image.hashCode()) ;
    cache_.put(id, image) ;
    return id;
  }
  
  public BufferedImage getChartByID(String id) throws Exception {
    BufferedImage image = (BufferedImage) cache_.get(id);
    if (image != null ){ cache_.remove(id); }
    return image;
  }
  
  private BufferedImage createPieChart(Chart chart) {
    DefaultPieDataset dataset = new DefaultPieDataset();
    Iterator it =  chart.getChartData().iterator();
    JFreeChart backChart;    
    while(it.hasNext()) {
      Data data = (Data)it.next();
      dataset.setValue(data.getType(), data.getValue());
    }    
    //Create chart
    if(chart.getIs3DChart()){
      backChart = ChartFactory.createPieChart3D(chart.getChartTitle(),dataset,true, true, false);      
    }else{
      backChart = ChartFactory.createPieChart(chart.getChartTitle(),dataset,true, true, false);
    }
    backChart.setBackgroundPaint(chart.getBackGroundColor());
    PiePlot plot = (PiePlot)backChart.getPlot();    
    //Start point
    plot.setStartAngle(chart.getStartAngle());
    //Display color
    it = chart.getChartData().iterator();
    while(it.hasNext()){
      Data data = (Data)it.next();
      if(data.getDisplayColor()!= null) plot.setSectionPaint(data.getPainPosition(), data.getDisplayColor());
    }
    //Direct rotation
    plot.setDirection(chart.getDirection());
    //Foreground
    if (chart.getIsForeGround()) plot.setForegroundAlpha(0.5f);
    return backChart.createBufferedImage(chart.getChartWidth(), chart.getChartHeight());
  }
  
  private BufferedImage createBarChart(Chart chart) {
    DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    PlotOrientation orientation;
    JFreeChart backChart;
    Iterator it =  chart.getChartData().iterator();
    while(it.hasNext()) {
      Data data = (Data)it.next();
      dataset.setValue(data.getValue(), data.getSeries(), data.getType());      
    }    
    //Set chart Orientation
    if(chart.getIsHorizontal()) { orientation = PlotOrientation.HORIZONTAL; }
    else{ orientation = PlotOrientation.VERTICAL; }
    //Create chart
    if(chart.getIs3DChart()) {
      backChart = ChartFactory.createBarChart3D(chart.getChartTitle(), chart.getCategoryLabel(), chart.getValueLabel(), dataset, orientation, true, true, false); 
    }else {
      backChart = ChartFactory.createBarChart(chart.getChartTitle(), chart.getCategoryLabel(), chart.getValueLabel(), dataset, orientation, true, true, false);
    }
    CategoryPlot plot = backChart.getCategoryPlot();
    CategoryItemRenderer renderer = plot.getRenderer();
    if (chart.getIsDisplayItemLabel()) {
      renderer.setItemLabelGenerator(new ExoCategoryItemLabelGenerator());
      renderer.setItemLabelsVisible(true);
    }
    // Set display color for bar
    it = chart.getChartData().iterator();
    while(it.hasNext()){
      Data data = (Data)it.next();
      if(data.getDisplayColor()!= null) renderer.setSeriesPaint(data.getPainPosition(), new GradientPaint( 0.0f, 0.0f, data.getDisplayColor(), 0.0f, 0.0f, data.getDisplayColor()));
    }
    return backChart.createBufferedImage(chart.getChartWidth(), chart.getChartHeight());    
  }
 }
