/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.chart.test;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.jmx.JMXProxyComponent;
import org.exoplatform.services.chart.Chart;
import org.exoplatform.services.chart.ChartService;
import org.exoplatform.services.chart.Data;
import org.exoplatform.test.BasicTestCase;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageEncoder;


/**
 * Created by The eXo Platform SARL
 * Author : Hung Nguyen
 *          nqhungvn@users.sourceforge.net
 * Jul 18, 2005
 */

public class TestChartServiceImpl extends BasicTestCase {
	private ChartService service_;
	public TestChartServiceImpl(String name) {
	  super(name);
	}
	public void setUp() throws Exception {
    PortalContainer pcontainer = PortalContainer.getInstance() ;
		service_ = 
      (ChartService)pcontainer.getComponentInstanceOfType(ChartService.class) ;
    
    MBeanServer server = pcontainer.getMBeanServer() ;
    ObjectName name = new ObjectName("chart:type=ChartService") ;
    service_ = 
        (ChartService)JMXProxyComponent.getComponent(ChartService.class, server, name);
	}
  
  public void testChartService() throws Exception {
		int width = 600;
		int height = 400;
    String filename;
    String id;
    Chart chart;
    //Create pie chart default color
    chart = createPieChart(width, height, false);
    filename = "./target/piechartDefaultColor.jpg";
    saveToImage(chart, filename, width, height);
    
    //Create 3D pie chart and set display color {cyan ; green ; yellow ; lightgray}
    chart = createPieChart(width, height, true);
    chart.setIs3DChart(true);
    filename = "./target/piechart3D.jpg";
    saveToImage(chart, filename, width, height);
    
    //Create 3D pie chart with foreGround and set display color {cyan ; green ; yellow ; lightgray}
    chart = createPieChart(width, height, true);
    chart.setIs3DChart(true);
    chart.setIsForeGround(true);
    filename = "./target/piechart3DForeground.jpg";
    saveToImage(chart, filename, width, height);
    
    //Create Vectical Bar chart default color
    chart = createBarChart(width, height, false);
    filename = "./target/barChartDefaultColor.jpg";
    saveToImage(chart, filename, width, height);
    
    //Create Horizontal Bar chart
    chart = createBarChart(width, height, true);
    chart.setIsHorizontal(true);
    chart.setIsDisplayItemLabel(true) ;
    filename = "./target/barChartHorizontal.jpg";
    saveToImage(chart, filename, width, height);
    
    //Create 3D Vectical Bar chart
    chart = createBarChart(width, height, true);
    chart.setIs3DChart(true);
    chart.setIsDisplayItemLabel(true) ;
    filename = "./target/barChart3D.jpg";
    saveToImage(chart, filename, width, height);
    
    //Create 3D Horizontal Bar chart
    chart = createBarChart(width, height, true);
    chart.setIs3DChart(true);
    chart.setIsHorizontal(true);
    id = saveToCache(chart);    
    if (id != null && id.length() != 0) {
      filename = "./target/barChart3DHorizontalFromCache.jpg";
      BufferedImage ret = service_.getChartByID(id);
      assertTrue("Chart Width", ret.getWidth() == width) ;
      assertTrue("Chart Heitht", ret.getHeight() == height) ;
      assertTrue("Return value from chart is not null", ret != null) ;
      OutputStream output = new FileOutputStream(new File(filename));
      JPEGImageEncoder pp = JPEGCodec.createJPEGEncoder(output);
      pp.encode(ret);
      output.close();
    }     
	}
  
	protected String getDescription() {
	  return "Test Simple chart Service" ;
	}
  
  private Chart createPieChart(int width, int height, boolean newColor){
    Chart chart = new Chart();
    ArrayList dataset = createData(newColor);
    chart.setChartTitle("Hello! I'm Pie chart");
    chart.setChartType(Chart.PIE_CHART);
    chart.setIs3DChart(false);
    chart.setIsForeGround(false);
    chart.setChartWidth(width);
    chart.setChartHeight(height);
    chart.setChartData(dataset);
    return chart;
  }
  private Chart createBarChart(int width, int height, boolean newColor){
    Chart chart = new Chart();
    ArrayList dataset = createBarData(newColor);
    chart.setChartTitle("Hello! I'm Bar chart");
    chart.setChartType(Chart.COLUMN_CHART);
    chart.setIs3DChart(false);
    chart.setChartWidth(width);
    chart.setChartHeight(height);
    chart.setCategoryLabel("Category");
    chart.setValueLabel("Value");
    chart.setIsHorizontal(false);
    chart.setChartData(dataset);
    chart.getCategoryLabel() ; 
    return chart;
  }
  
  private String saveToCache(Chart chart)throws Exception {
    String id = service_.getChartID(chart);
    return id;
  }
  private BufferedImage getChartFromCache(String id) throws Exception {
    BufferedImage image = service_.getChartByID(id);
    return image;
  }
  private void saveToImage(Chart chart, String filename, int width, int height) throws Exception {
    BufferedImage ret = service_.createChart(chart);
    assertTrue("Chart Width", ret.getWidth() == width) ;
    assertTrue("Chart Heitht", ret.getHeight() == height) ;
    assertTrue("Return value from chart is not null", ret != null) ;
    OutputStream output = new FileOutputStream(new File(filename));
    JPEGImageEncoder pp = JPEGCodec.createJPEGEncoder(output);
    pp.encode(ret);
    output.close();
  }  
  private ArrayList createBarData(boolean newColor) {
    ArrayList dataset = new ArrayList();    
    Data data = new Data();
    data.setValue(0);
    data.setSeries("series1");
    data.setType("Ha noi");
    if (newColor){
      data.setDisplayColor(Color.cyan);
      data.setPainPosition(0);
    }
    dataset.add(data);    
    data = new Data();
    data.setValue(0);
    data.setSeries("series2");
    data.setType("Ha noi");
    
    if (newColor){
      data.setDisplayColor(Color.green);
      data.setPainPosition(1);
    }
    dataset.add(data);    
    data = new Data();
    data.setValue(15.0);
    data.setSeries("series3");
    data.setType("Ha noi");
    if (newColor){
      data.setDisplayColor(Color.yellow);
      data.setPainPosition(2);
    }
    dataset.add(data);    
    data = new Data();
    data.setValue(17);
    data.setSeries("series4");
    data.setType("Ha noi");
    if (newColor){
      data.setDisplayColor(Color.lightGray);
      data.setPainPosition(3);
    }
    dataset.add(data);    
    data = new Data();
    data.setValue(35.0);
    data.setSeries("series1");
    data.setType("TPHCM");
    if (newColor){
      data.setDisplayColor(Color.cyan);
      data.setPainPosition(0);
    }
    dataset.add(data);     
    data = new Data();
    data.setValue(23.0);
    data.setSeries("series2");
    data.setType("TPHCM");
    if (newColor){
      data.setDisplayColor(Color.green);
      data.setPainPosition(1);
    }
    dataset.add(data);     
    data = new Data();
    data.setValue(19.0);
    data.setSeries("series3");
    data.setType("TPHCM");
    if (newColor){
      data.setDisplayColor(Color.yellow);
      data.setPainPosition(2);
    }
    dataset.add(data);    
    data = new Data();
    data.setValue(27);
    data.setSeries("series4");
    data.setType("TPHCM");
    if (newColor){
      data.setDisplayColor(Color.lightGray);
      data.setPainPosition(3);
    }
    dataset.add(data);    
    return dataset;
  }
  private ArrayList createData(boolean newColor) {
    ArrayList dataset = new ArrayList();
    Data data = new Data();
    data.setValue(30.0);
    data.setSeries("series1");
    data.setType("Ha noi");
    if(newColor){
      data.setDisplayColor(Color.cyan);
      data.setPainPosition(0);
    }
    dataset.add(data);
    data = new Data();
    data.setValue(28.0);
    data.setSeries("series2");
    data.setType("TPHCM");
    if(newColor){
      data.setDisplayColor(Color.green);
      data.setPainPosition(1);
    }
    dataset.add(data);
    data = new Data();
    data.setValue(15.0);
    data.setSeries("series3");
    data.setType("Hai phong");
    if(newColor){
      data.setDisplayColor(Color.yellow);
      data.setPainPosition(2);      
    }
    dataset.add(data);
    data = new Data();
    data.setValue(17);
    data.setSeries("series4");
    data.setType("Da nang");
    if(newColor){
      data.setDisplayColor(Color.lightGray);
      data.setPainPosition(3);      
    }
    dataset.add(data);
    return dataset;
  }
}
