/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization.ldap;

import java.util.Date;

import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttributes;

import org.exoplatform.services.ldap.ObjectClassAttribute;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserHandler;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.services.organization.impl.GroupImpl;
import org.exoplatform.services.organization.impl.MembershipTypeImpl;
import org.exoplatform.services.organization.impl.UserProfileData;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Oct 13, 2005
 */
public class LDAPAttributeMapping {
  
   static String[] USER_LDAP_CLASSES  =  {"top", "person", "organizationalPerson","inetOrgPerson" } ;
   static String[]  PROFILE_LDAP_CLASSES  =  {"top", "organizationalPerson"} ;
   static String[]  GROUP_LDAP_CLASSES  =  {"top", "organizationalUnit"} ;
   static String[]  MEMBERSHIPTYPE_LDAP_CLASSES  =  {"top", "organizationalRole"} ;
   static String[]  MEMBERSHIP_LDAP_CLASSES  =  { "top",  "groupOfNames"} ;  
  
   String baseURL = "dc=exoplatform,dc=org" ;
   String groupsURL ="ou=groups,ou=portal,dc=exoplatform,dc=org" ;
   String membershipTypeURL ="ou=memberships,ou=portal,dc=exoplatform,dc=org" ;  
   String userURL = "ou=users,ou=portal,dc=exoplatform,dc=org" ;
   String profileURL = "ou=profiles,ou=portal,dc=exoplatform,dc=org" ;
  
   String userAuthenticationAttr ="uid" ;
   String userUsernameAttr ="uid" ;
   String userPassword ="userPassword"  ;
   String userFirstNameAttr = "givenName"  ;
   String userLastNameAttr ="sn"  ;
   String userDisplayNameAttr = "displayName" ;
   String userMailAttr ="mail" ;
   String userObjectClassFilter = "objectClass=person" ;
  
   String membershipTypeMemberValue = "member";
   String membershipTypeRoleNameAttr = "cn"  ;
   String membershipTypeNameAttr = "cn"  ;
   String membershipTypeObjectClassFilter = "objectClass=organizationalRole" ;
   String membershiptypeObjectClass = "organizationalRole" ;
  
   String groupObjectClass = "organizationalUnit"  ;
   String groupObjectClassFilter = "objectClass=organizationalUnit" ;
  
   String membershipObjectClass ="groupOfNames" ;
   String membershipObjectClassFilter ="objectClass=groupOfNames"  ;
  
   String ldapCreatedTimeStampAttr = "createdTimeStamp"  ;
   String ldapModifiedTimeStampAttr = "modifiedTimeStamp"  ;
   String ldapDescriptionAttr = "description"  ;
  
  final public Attributes  userToAttributes(User user) {
    BasicAttributes attrs = new BasicAttributes();
    attrs.put( new ObjectClassAttribute( USER_LDAP_CLASSES));
    attrs.put( "cn", user.getUserName());
    attrs.put( userDisplayNameAttr, user.getFullName());
    attrs.put( userUsernameAttr, user.getUserName());
    attrs.put( userPassword, user.getPassword());
    attrs.put( userLastNameAttr, user.getLastName());
    attrs.put( userFirstNameAttr, user.getFirstName());
    attrs.put( userMailAttr, user.getEmail());
    attrs.put( ldapDescriptionAttr, "Account for "+user.getFullName());
    return attrs;
  }
  
  final public User attributesToUser(UserHandler handler, Attributes attrs) throws Exception {
    if(attrs == null ||  attrs.size() == 0)  return null ;
    User user =  handler.createUserInstance() ;
    user.setUserName( getAttributeValueAsString( attrs, userUsernameAttr)) ;
    user.setLastName( getAttributeValueAsString( attrs, userLastNameAttr)) ;
    user.setFirstName( getAttributeValueAsString( attrs, userFirstNameAttr)) ;
    user.setFullName( getAttributeValueAsString( attrs, userDisplayNameAttr)) ;
    user.setEmail( getAttributeValueAsString( attrs, userMailAttr)) ;    
    user.setPassword( getAttributeValueAsString( attrs, userPassword)) ;  
    user.setCreatedDate(new Date());
    user.setLastLoginTime(new Date());
    return user ;
  }  
  
  final public Attributes groupToAttributes(Group group) {
    BasicAttributes attrs = new BasicAttributes();
    attrs.put(new ObjectClassAttribute( GROUP_LDAP_CLASSES));
    attrs.put("ou", group.getGroupName());
    String desc = group.getDescription();
    if (desc != null && desc.length() > 0) attrs.put("description", desc);
    String lbl = group.getLabel();
    if( lbl != null && lbl.length() > 0)  attrs.put("l", lbl);
    return attrs;
  }
  
  final public Group attributesToGroup( Attributes attrs) throws Exception {
    if(attrs == null ||  attrs.size() == 0)  return null ;
    Group group = new GroupImpl();    
    group.setGroupName( getAttributeValueAsString( attrs, "ou"));
    group.setDescription( getAttributeValueAsString( attrs, "description"));    
    group.setLabel( getAttributeValueAsString( attrs, "l"));    
    return group;    
  }
  
  final public Attributes membershipTypeToAttributes(MembershipType mt) {    
    BasicAttributes attrs = new BasicAttributes();
    attrs.put(new ObjectClassAttribute( MEMBERSHIPTYPE_LDAP_CLASSES));  
    attrs.put( membershipTypeNameAttr, mt.getName());    
    String desc = mt.getDescription();
    if (desc != null && desc.length() > 0) attrs.put( "description", desc);        
    return attrs;
  }
  
  final public MembershipType attributesToMembershipType( Attributes attrs){
    if(attrs == null ||  attrs.size() == 0)  return null ;
    MembershipType m = new MembershipTypeImpl();
    m.setName(  getAttributeValueAsString(attrs, membershipTypeNameAttr));
    m.setDescription( getAttributeValueAsString(attrs, "description"));
    m.setCreatedDate(new Date());
    m.setModifiedDate(new Date());
    return m;
  }
  
  final public Attributes membershipToAttributes(Membership m, String userDN) {
    BasicAttributes attrs = new BasicAttributes();
    attrs.put( new ObjectClassAttribute( MEMBERSHIP_LDAP_CLASSES));  
    attrs.put(  membershipTypeRoleNameAttr, m.getMembershipType());
    attrs.put( membershipTypeMemberValue, userDN);    
    return attrs;
  }
  
  final public String getAttributeValueAsString(Attributes attributes, String name) {   
    if (attributes == null)  return "";
    Attribute attr = attributes.get(name);
    if (attr == null) return "";
    try {
      Object obj =  attr.get();
      if(obj instanceof byte[]) return new String((byte[]) obj) ;
      return (String) obj ;
    } catch (Exception e) {
      e.printStackTrace();
      return "";
    }   
  }
  
  final public Attributes profileToAttributes( UserProfile profile) {
    BasicAttributes attrs = new BasicAttributes();
    attrs.put(new ObjectClassAttribute( PROFILE_LDAP_CLASSES));
    attrs.put("sn", profile.getUserName());
    UserProfileData upd = new UserProfileData() ;  
    upd.setUserProfile( profile) ;    
    attrs.put( ldapDescriptionAttr, upd.getProfile());
    return attrs;
  }
  
  final public UserProfileData attributesToProfile( Attributes attrs) {   
    if(attrs == null ||  attrs.size() == 0)  return null ;
    UserProfileData upd = new UserProfileData() ;    
    upd.setProfile( getAttributeValueAsString(attrs, ldapDescriptionAttr));    
    return upd;
  }
}
