/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.jmx.component;

import javax.faces.component.UIComponent;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.exoplatform.container.SessionManager;
import org.exoplatform.container.jmx.JMXProxyComponent;
import org.exoplatform.container.monitor.jvm.JVMRuntimeInfo;
import org.exoplatform.container.monitor.jvm.OperatingSystemInfo;
import org.exoplatform.faces.core.component.UINode;
import org.exoplatform.portlets.cache.component.UICachesChart;
import org.exoplatform.portlets.cache.component.UIListCache;
import org.exoplatform.portlets.jvm.component.UIJVMRuntimeInfo;
import org.exoplatform.portlets.jvm.component.UIOSInfo;
import org.exoplatform.portlets.log.component.UILog;
import org.exoplatform.portlets.log.component.UILogConfig;
import org.exoplatform.portlets.log.component.UILogError;
import org.exoplatform.portlets.monitor.component.UILiveSessions;
import org.exoplatform.portlets.monitor.component.UIPortalActivitiesLogChart;
import org.exoplatform.portlets.monitor.component.UIStatisticsPortalActionLog;
import org.exoplatform.portlets.monitor.component.UIStatisticsUserActionLog;
import org.exoplatform.portlets.monitor.component.UIUserActivitiesLogChart;
import org.exoplatform.portlets.remote.component.UIClusterInfo;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.chart.ChartService;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.portal.log.PortalActivitiesLogService;
import org.exoplatform.services.portal.log.UserActivitiesLogService;
import org.exoplatform.services.remote.group.CommunicationService;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIMBean.java,v 1.3 2004/08/01 18:56:11 tuan08 Exp $
 */
public class UIAdvancedViewMBean extends UINode {
	public UIAdvancedViewMBean() {
	  setRendererType("ChildrenRenderer") ;
	}
  
	public boolean  setData(MBeanServer server, ObjectName name) throws Exception {
    getChildren().clear() ;
    setRendererType("ChildrenRenderer") ;
	  if("monitor:type=LogService".equals(name.getCanonicalName())) {
      return addLogNode(server, name) ;
    } else if("cache:type=CacheService".equals(name.getCanonicalName())) {
      return addCacheNode(server, name) ;
    } else if("monitor:type=PortalActivitiesLogService".equals(name.getCanonicalName())) {
      return addPortalMonitors(server, name) ;
    }else if("monitor:type=UserActivitiesLogService".equals(name.getCanonicalName())) {
      return addUserMonitors(server, name) ;
    } else if("monitor:type=SessionManager".equals(name.getCanonicalName())) {
      return addSessionManager(server, name) ;
    } else if("info:type=JVMRuntimeInfo".equals(name.getCanonicalName())) {
      return addJVMRuntimeInfo(server, name) ;
    } else if("info:type=OperatingSystemInfo".equals(name.getCanonicalName())) {
      return addOSInfo(server, name) ;
    } else if("remote:type=CommunicationService".equals(name.getCanonicalName())) {
      return addCommunicationService(server, name) ;
    }
    return false ;
	}
  
  private boolean addPortalMonitors(MBeanServer server, ObjectName name) throws Exception {
    PortalActivitiesLogService service =   
      (PortalActivitiesLogService)JMXProxyComponent.getComponent(PortalActivitiesLogService.class, server, name);
    addChild(new UIPortalActivitiesLogChart(), false);
    addChild(new UIStatisticsPortalActionLog(), true) ;
    setRendererType("VelocityNodeMenuRenderer");
    setTemplatePath("org/exoplatform/faces/core/renderer/html/UISecondTabLevel.vm");
    return true ;
  }
  
  private boolean addUserMonitors(MBeanServer server, ObjectName name) throws Exception {
    UserActivitiesLogService service =   
      (UserActivitiesLogService)JMXProxyComponent.getComponent(UserActivitiesLogService.class, server, name);
    addChild(new UIUserActivitiesLogChart(), false) ;
    addChild(new UIStatisticsUserActionLog(), true);
    setRendererType("VelocityNodeMenuRenderer");
    setTemplatePath("org/exoplatform/faces/core/renderer/html/UISecondTabLevel.vm");
    return true ;
  }
  private boolean addLogNode(MBeanServer server, ObjectName name) throws Exception {
    LogService service = 
      (LogService)JMXProxyComponent.getComponent(LogService.class, server, name);
    addChild(new UILog(service), true);
    addChild(new UILogError(service), false);
    addChild(new UILogConfig(service), false);
    setRendererType("VelocityNodeMenuRenderer");
    setTemplatePath("org/exoplatform/faces/core/renderer/html/UISecondTabLevel.vm");
    return true ;
  }
  
  private boolean addCacheNode(MBeanServer server, ObjectName name) throws Exception{
    setRendererType("ChildrenRenderer");
    CacheService service = 
      (CacheService)JMXProxyComponent.getComponent(CacheService.class, server, name);
    name = new ObjectName("chart:type=ChartService") ;
    ChartService chartService = 
      (ChartService)JMXProxyComponent.getComponent(ChartService.class, server, name);
    addChild(new UIListCache(service), true);
    addChild(new UICachesChart(chartService), false);
    return true ;
  }
  
  private boolean addSessionManager(MBeanServer server, ObjectName name) throws Exception{
    setRendererType("ChildrenRenderer");
    SessionManager service = 
      (SessionManager)JMXProxyComponent.getComponent(SessionManager.class, server, name);
    addChild(new UILiveSessions(service), true);   
    return true ;
  }
  
  private boolean addJVMRuntimeInfo(MBeanServer server, ObjectName name) throws Exception{
    setRendererType("ChildrenRenderer");
    JVMRuntimeInfo service = 
      (JVMRuntimeInfo)JMXProxyComponent.getComponent(JVMRuntimeInfo.class, server, name);
    addChild(new UIJVMRuntimeInfo(service), true);   
    return true;
  }
  
  private boolean addOSInfo(MBeanServer server, ObjectName name) throws Exception{
    setRendererType("ChildrenRenderer");
    OperatingSystemInfo service = 
      (OperatingSystemInfo)JMXProxyComponent.getComponent(OperatingSystemInfo.class, server, name);
    addChild(new UIOSInfo(service), true);   
    return true ;
  }
  
  private boolean addCommunicationService(MBeanServer server, ObjectName name) throws Exception{
    setRendererType("ChildrenRenderer");
    CommunicationService service = 
      (CommunicationService)JMXProxyComponent.getComponent(CommunicationService.class, server, name);
    addChild(new UIClusterInfo(service), true);   
    return  true ;
  }
  
  private void addChild(UIComponent comp, boolean render) {
    comp.setRendered(render) ;
    getChildren().add(comp) ;
  }
}