package org.exoplatform.portlets.monitor.component;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.monitor.component.UIUserActivitiesLogChart.ChartParameter;
import org.exoplatform.services.chart.Chart;
import org.exoplatform.services.chart.ChartService;
import org.exoplatform.services.chart.Data;
/**
 * wen, Aug 10, 2005 @ 13:07:02 
 * @author: Hung Nguyen
 * @email:   nqhungvn@users.sourceforge.net
 * @version: $Id: UIChart.java,v 1.14 2005/08/10 13:07:02 nqhungvn Exp $
 */
public class UIPortalActivitiesLogChart extends UIExoCommand {
	
	final public static Parameter[] BACK_PARAMS = {new Parameter(ACTION , CANCEL_ACTION) } ;

	private String portal_ ;
  private List listChartIdParam_ ;
  private boolean msgError_ = false ; 
	public UIPortalActivitiesLogChart() throws Exception {
    PortalContainer pcontainer =  PortalContainer.getInstance() ;
    portal_ = pcontainer.getPortalContainerInfo().getContainerName() ;    
    setId("UIPortalActionChart") ;
		setRendererType("VelocityRenderer") ;
    listChartIdParam_ = new ArrayList() ;
    getChildren().add(new UIPortalActionChartFilter());
		addActionListener(BackActionListener.class, CANCEL_ACTION) ;
    UIStatisticsPortalActionLog uiStatistics =  new UIStatisticsPortalActionLog() ;
    List logList = uiStatistics.getChartData(uiStatistics.getSelectType());
    updatePortalLogDataSet(logList, uiStatistics.getSelectType(),"") ;    
	}
	public Parameter[] getBackParams() { return BACK_PARAMS ; }
  public List getChartIds() {
    if(listChartIdParam_.size() > 0){       
      return listChartIdParam_ ;
    }else {
      ChartParameter nullChartId = new ChartParameter("null") ;
      List temp = new ArrayList() ;
      temp.add(nullChartId) ;
      return temp ;
    }     
  }
	
	public String getPortalName() { return portal_ ; }
  
	public void updatePortalLogDataSet( List dataList, String type, String mostAccess) throws Exception {
    PortalContainer pcontainer =  PortalContainer.getInstance() ;
    ChartService chartService = 
      (ChartService) pcontainer.getComponentInstanceOfType(ChartService.class) ;
    listChartIdParam_ = new ArrayList() ;
    portal_ = pcontainer.getPortalContainerInfo().getContainerName() ;
    ArrayList dataset = new ArrayList();
    ArrayList datasetAvg = new ArrayList();
    Data data ;
    int width = 600 ;
    int height = 400;
    String chartId ;
    Chart chart ;
    String title ;
    ChartParameter chartParam ;
    if(type.equals("portlet")){
      int size = dataList.size() ;
      if(mostAccess != "" && size > 0) 
        size = 1 ;
      for (int i=0; i < size; i++) {
        Object[] row = (Object[])dataList.get(i);
        String name = (String)row[0] ;
        Long accessCounter     = (Long)row[1] ;
        Long errorCounter      = (Long)row[2] ;
        Long summaryAccessTime = (Long)row[3] ;
        Long actionCounter      = (Long)row[4] ;
        Long errorActionCounter = (Long)row[5] ;
        Long summaryActionTime  = (Long)row[6] ;
        if(name.length() > 22){
          name = name.substring(0, 18) + "..." ;
        }
        //Data for counter char
        data = createData(accessCounter.longValue(), "Render count", name) ;
        dataset.add(data) ;
        data = createData(errorCounter.longValue(), "Render error", name) ;
        dataset.add(data) ;
        data = createData(actionCounter.longValue(), "Action count", name) ;
        dataset.add(data) ;
        data = createData(errorActionCounter.longValue(), "Action error", name) ;
        dataset.add(data) ;
        
        // data for AVG chart
        double avgAccessTime = 0;
        if(accessCounter.longValue() > 0 && summaryAccessTime.longValue() > 0)
          avgAccessTime = summaryAccessTime.longValue() / accessCounter.longValue() ;
        data = createData(avgAccessTime, "Avg render time", name) ;
        datasetAvg.add(data) ;  
        double avgActionTime = 0;
        data = createData(avgActionTime, "Avg action time", name) ;
        if(actionCounter.longValue() > 0 && summaryActionTime.longValue() > 0 )
          avgActionTime = summaryActionTime.longValue() / actionCounter.longValue() ;
        datasetAvg.add(data) ;       
      }      
      //create chart
      title = "eXoplatform Portlets Activities Render-Action Chart" ;
      chart = createChart(width, height, title, "Render counter / Action counter - Values", dataset);
      chartId = chartService.getChartID(chart) ;
      if(chartId == null) chartId = "null" ;
      chartParam = new ChartParameter(chartId) ;
      listChartIdParam_.add(chartParam) ;
      
      title = "eXoplatform Portlets Activities Average Access Chart" ;
      chart = createChart(width, height, title, "Average access time(ms) ", datasetAvg);
      chartId = chartService.getChartID(chart) ;
      if(chartId == null) chartId = "null" ;
      chartParam = new ChartParameter(chartId) ;
      listChartIdParam_.add(chartParam) ;        
    }else{
      int size = dataList.size() ;
      if(mostAccess != "" && size > 0) 
        size = 1 ;        
      for (int i=0; i < size; i++) {
        Object[] row = (Object[])dataList.get(i);
        String name = (String)row[0] ;
        Long accessCounter     = (Long)row[1] ;
        Long errorCounter      = (Long)row[2] ;
        Long summaryAccessTime = (Long)row[3] ; 
        if(name.length() > 22){
          name = name.substring(0, 18) + "..." ;
        }
        data = createData(accessCounter.longValue(), "Access counter", name) ;
        dataset.add(data) ;
        data = createData(errorCounter.longValue(), "Access error", name) ;
        dataset.add(data) ;        
        data = createData(summaryAccessTime.longValue() / accessCounter.longValue(), "Avg time", name) ;
        datasetAvg.add(data) ; 
        if(i >= 25) {
          msgError_ = true ;
          break ;
        }
      }       
      //create chart
      if(type.equals("page")) title = "eXoplatform Page Activities Access Chart" ;
      else title = "eXoplatform Portal Activities Access Chart" ;
            
      chart = createChart(width, height, title, "Access counter / Access error - Values", dataset);
      portal_ = pcontainer.getPortalContainerInfo().getContainerName() ;
      chartId = chartService.getChartID(chart) ;
      if(chartId == null) chartId = "null" ;
      chartParam = new ChartParameter(chartId) ;
      listChartIdParam_.add(chartParam) ;
      
      if(type.equals("page")) title = "eXoplatform Page Activities Average Access Chart" ;
      else title = "eXoplatform Portal Activities Average Access Chart" ;
      chart = createChart(width, height, title, "Average access time(ms) ", datasetAvg);
      chartId = chartService.getChartID(chart) ;
      if(chartId == null) chartId = "null" ;
      chartParam = new ChartParameter(chartId) ;
      listChartIdParam_.add(chartParam) ;       
      ChartParameter param = (ChartParameter)listChartIdParam_.get(0) ;
    }       
  } 
  public boolean getMsg(){ return msgError_ ; }
  private Data createData(double value, String serie, String type){
    Data data = new Data() ;
    data.setValue(value) ;
    data.setSeries(serie) ;
    data.setType(type) ;
    return data ;
  }
  private Chart createChart(int width, int height, String title, String valueLabel, ArrayList dataset) {
    Chart chart = new Chart();
    chart.setChartTitle(title);
    chart.setChartType(Chart.COLUMN_CHART);
    chart.setIs3DChart(false);
    chart.setChartWidth(width);
    chart.setChartHeight(height);
    chart.setCategoryLabel("Name");
    chart.setValueLabel(valueLabel);
    chart.setIsHorizontal(true);
    chart.setIsDisplayItemLabel(true) ;
    chart.setChartData(dataset);
    return chart ;
  }
	static public class BackActionListener extends ExoActionListener  {
		public void execute(ExoActionEvent event) throws Exception {
			UIPortalActivitiesLogChart uiChart = (UIPortalActivitiesLogChart) event.getComponent() ;
			String baseURL = uiChart.getBaseURL() ;
			uiChart.setRenderedSibling(uiChart.getSibling(UIStatisticsPortalActionLog.class).getClass()) ;
		}
	}  
}
