/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.faces.core.component.UINode;
import org.exoplatform.faces.core.component.UICommandNode;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.OrganizationService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIGroupExplorer.java,v 1.15 2004/10/22 14:23:30 tuan08 Exp $
 */
public class UIGroupExplorer extends UICommandNode {
  static public String GROUP_ID = "groupId" ;
  static public String CHANGE_GROUP =  "changeGroup" ;
  static public String ADD_GROUP =  "addGroup" ;
  static public String DELETE_GROUP =  "deleteGroup" ;
  static Parameter[] CHANGE_GROUP_PARAMS = { new Parameter(ACTION , CHANGE_GROUP)} ;
  static Parameter[] ADD_GROUP_PARAMS = { new Parameter(ACTION , ADD_GROUP)} ;
  static Parameter[] DELETE_GROUP_PARAMS = { new Parameter(ACTION , DELETE_GROUP)} ;
  
  private OrganizationService service_ ; 
  private Group currentGroup_ ;
  private Group parentGroup_ ;
  private Collection childrenGroup_ ;
  private boolean adminRole_ ;
   
  public UIGroupExplorer(OrganizationService service) throws Exception {
  	setId("UIGroupExplorer") ;
    setRendererType("VelocityRenderer") ;
    service_ = service ;
    customizeGroupDetail() ;
    childrenGroup_ = service_.getGroupHandler().findGroups(null) ;
    addActionListener(ChangeGroupActionListener.class, CHANGE_GROUP) ;
    addActionListener(AddGroupActionListener.class, ADD_GROUP) ;
    addActionListener(DeleteGroupActionListener.class, DELETE_GROUP) ;
    adminRole_ = hasRole("admin") ;
  }
  
  public Parameter[]  getChangeGroupParams()  { return CHANGE_GROUP_PARAMS ; }
  public Parameter[]  getAddGroupParams()  { return ADD_GROUP_PARAMS ; }
  public Parameter[]  getDeleteGroupParams()  { return DELETE_GROUP_PARAMS ; }
  
  protected void customizeGroupDetail() throws Exception {
    UINode uiGroupDetail = (UINode)addChild(UINode.class) ;
    uiGroupDetail.setId("UIGroupDetail") ;
    uiGroupDetail.setRendererType("VelocityNodeMenuRenderer") ;
    uiGroupDetail.setTemplatePath("org/exoplatform/faces/core/renderer/html/UISecondTabLevel.vm") ;
    
    uiGroupDetail.addChild(UIViewUserInGroup.class) ;
    
    UINode uiCommunityNode = (UINode)uiGroupDetail.addChild(UINode.class) ;
    uiCommunityNode.setId("UICommunityNode") ;
    uiCommunityNode.setRendererType("ChildrenRenderer") ;
    uiCommunityNode.setRendered(false);
    uiCommunityNode.addChild(UIGroupCommunityInfo.class) ;
    uiCommunityNode.addChild(UICommunityPortalForm.class).setRendered(false) ;
    uiCommunityNode.addChild(UICommunityNavForm.class).setRendered(false) ;
  }
  
  public boolean hasAdminRole() { return adminRole_ ; } 
  public Group getCurrentGroup() { return currentGroup_ ; }
  public Group getParentGroup() { return parentGroup_ ; }
  public Collection getChildrenGroup() { return childrenGroup_ ; }
  
  public PageList getMemberOfTheCurrentGroup() throws Exception {
    if(currentGroup_ == null) return PageList.EMPTY_LIST ; 
    return service_.getUserHandler().findUsersByGroup(currentGroup_.getId());
  }
  public List getParentsGroup(Group group) throws Exception{
    List parentGroupList  = new ArrayList();
    Group currentGroup = group ;
    while(currentGroup != null){
     Group parentGroup = service_.getGroupHandler().findGroupById(currentGroup.getParentId()) ;
     if(parentGroup != null) parentGroupList.add(parentGroup) ;     
     currentGroup = parentGroup ;
    }
    parentGroupList.add(group) ;
    return parentGroupList ;
  }
  public void update() throws Exception {
    if(currentGroup_ != null ) {
      currentGroup_  = service_.getGroupHandler().findGroupById(currentGroup_.getId()) ;
      parentGroup_ = service_.getGroupHandler().findGroupById(currentGroup_.getParentId()) ;
      childrenGroup_ = service_.getGroupHandler().findGroups(parentGroup_) ;
    } else {
      currentGroup_  = null ;
      childrenGroup_ = service_.getGroupHandler().findGroups(null) ;
      parentGroup_ = null  ;
    }
  }
  
  private void changeGroup(String groupId) throws Exception {
  	if(groupId != null && groupId.length() > 0) {
  		currentGroup_  = service_.getGroupHandler().findGroupById(groupId) ;
      if(currentGroup_.getParentId() != null) {
      	parentGroup_ = service_.getGroupHandler().findGroupById(currentGroup_.getParentId()) ;
      } else {
        parentGroup_ = null ; 
      }
  		childrenGroup_ = service_.getGroupHandler().findGroups(currentGroup_) ;
  	} else {
  		currentGroup_  = null ;
  		childrenGroup_ = service_.getGroupHandler().findGroups(null) ;
  		parentGroup_ = null  ;
  	}
    broadcastOnChange();
  }
  
  static public class ChangeGroupActionListener extends ExoActionListener  {
  	public void execute(ExoActionEvent event) throws Exception {
      UIGroupExplorer uiExplorer = (UIGroupExplorer) event.getComponent() ;
  		String groupName = event.getParameter(OBJECTID) ;
  		uiExplorer.changeGroup(groupName) ;
  	}
  }
  
  static public class AddGroupActionListener extends ExoActionListener  {
    public AddGroupActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
      UIGroupExplorer uiExplorer = (UIGroupExplorer) event.getComponent() ;
  		String parentGroupId = event.getParameter(OBJECTID) ;
  		UIGroupForm uiForm = 
  			(UIGroupForm)uiExplorer.getSibling(UIGroupForm.class) ;
  		uiForm.setParentGroup(parentGroupId) ;
  		uiExplorer.setRenderedSibling(UIGroupForm.class);
  	}
  }
  
  static public class DeleteGroupActionListener extends ExoActionListener  {
    public DeleteGroupActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
      UIGroupExplorer uiExplorer = (UIGroupExplorer) event.getComponent() ;
  		if(uiExplorer.currentGroup_ != null) {
  		  uiExplorer.service_.getGroupHandler().removeGroup(uiExplorer.currentGroup_, true) ;
  		  uiExplorer.currentGroup_ = uiExplorer.parentGroup_ ;
  		  uiExplorer.update() ;
  		}
  	}
  }
}