/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.log.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.component.BaseComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.ValueParam;
import org.exoplatform.services.cache.CacheListener;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.cache.FIFOExoCache;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.exoplatform.services.portal.log.PortalLogData;
import org.exoplatform.services.portal.log.Query;
import org.hibernate.Session;

/**
 * @author Hung Nguyen (nqhungvn@users.sourceforge.net)
 * @since Sep 26, 2005
 * @version $Id$
 */
public class PortalActivitiesLogPlugin extends BaseComponentPlugin implements CacheListener {
  private FIFOExoCache[] portal_ = new FIFOExoCache[24] ;
  private int currentTimeIndex_ ;
  private Date currentTimeSlot_ ;
  private HibernateService hservice_ ;
  private int cacheSize_ ;
  private String uniqueId_ ;
  
  public PortalActivitiesLogPlugin(HibernateService hservice, InitParams params) throws Exception {
    hservice_ = hservice ;
    ValueParam param = params.getValueParam("cachesize") ;
    cacheSize_ = Integer.parseInt(param.getValue());
    updateTime() ;    
  }
  
  public void  updateTime() {
    GregorianCalendar  calendar = new  GregorianCalendar() ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    currentTimeSlot_ = calendar.getTime() ;
    currentTimeIndex_ = calendar.get(GregorianCalendar.HOUR_OF_DAY) ;
    uniqueId_ = "/" + String.valueOf(currentTimeSlot_.getYear() + 1900) + 
                "-" + String.valueOf(currentTimeSlot_.getMonth() + 1) +
                "-" + String.valueOf(currentTimeSlot_.getDate()) +
                "/" + String.valueOf(currentTimeIndex_);
    portal_[currentTimeIndex_] = new FIFOExoCache(cacheSize_) ;
    portal_[currentTimeIndex_].addCacheListener(this) ; 
    System.out.println("Current time slot is : " + calendar.getTime()) ;    
  }
  public int getCurrentTimeIndex() { return currentTimeIndex_ ; }
  public void logPortalAccess(String portalId, String portalOwner, long exeTime, boolean error) {
    getOrCreatePortalLogData(portalId, portalOwner, exeTime, error);    
  }
  private void getOrCreatePortalLogData(String portalId, String portalOwner, long exeTime, boolean error) {
    try {
      PortalLogDataImpl impl = 
    	  (PortalLogDataImpl) portal_[currentTimeIndex_].get(portalOwner + uniqueId_) ;
      if(impl == null) {
        impl = new PortalLogDataImpl() ;
        impl.logPortalAccess(portalOwner + uniqueId_, portalId, portalOwner, currentTimeSlot_, 
                             currentTimeIndex_, exeTime, error) ;
        portal_[currentTimeIndex_].put(portalOwner + uniqueId_, impl) ;
      }else{
        impl.updatePortalAccess(exeTime, error) ;
      } 
      
    } catch(Exception ex) {
      ex.printStackTrace() ;      
    }
  }
  
  synchronized public void savePortalLogData(PortalLogData data) throws Exception {
    Session session = hservice_.openSession() ;
    PortalLogDataImpl impl = 
      (PortalLogDataImpl) session.get(PortalLogDataImpl.class, data.getId()) ;
    if(impl == null) {
      impl = (PortalLogDataImpl) data ;
      session.save(impl, impl.getId()) ;
    } else {
      impl.merge((PortalLogDataImpl)data) ;
      session.update(impl,impl.getId()) ;      
    }
    session.flush() ;       
  }
  
  public PageList getPortalLogDatas(Query q) throws Exception {
    ObjectQuery oq = new ObjectQuery(PortalLogDataImpl.class);
    oq.addGT("timeSlot", q.getFromTimeSlot()) ;
    oq.addLT("timeSlot", q.getToTimeSlot()) ;
    oq.addGT("accessCounter", q.getCounter()) ;
    String componentId = q.getComponentId() ; 
    if(!componentId.equals("") || componentId.length() > 0) 
      oq.addLIKE("portalOwner", componentId) ;
    if(q.getOrder() != "" && q.getOrder().equals("Desc")){
      oq.setDescOrderBy("timeSlot") ;
    }else if(q.getOrder() != "" && q.getOrder().equals("Asc")){
      oq.setAscOrderBy("timeSlot") ;
    }
    return new DBObjectPageList(hservice_, oq);
  }
  public List getPortalLogChartDatas(Query q) throws Exception {
    ObjectQuery oq = new ObjectQuery(PortalLogDataImpl.class);
    oq.addSelect("portalOwner") ;
    oq.addSUM("accessCounter") ;
    oq.addSUM("errorCounter") ;
    oq.addSUM("summaryAccessTime") ;
    if(q.getMostAccess() != "" && q.getMostAccess().equals("most24")) {
      GregorianCalendar calendar = new GregorianCalendar() ;
      oq.addLT("timeSlot", calendar.getTime()) ;
      int dateBefore = calendar.get(GregorianCalendar.DATE) - 1;
      calendar.set(GregorianCalendar.DATE, dateBefore) ;
      oq.addGT("timeSlot", calendar.getTime()) ;
    }else if (q.getMostAccess() != "" && q.getMostAccess().equals("overall")){
      GregorianCalendar calendar = new GregorianCalendar() ;
      oq.addLT("timeSlot", calendar.getTime()) ;
    }else {
      oq.addGT("timeSlot", q.getFromTimeSlot()) ;
      oq.addLT("timeSlot", q.getToTimeSlot()) ; 
    }
    oq.addGT("accessCounter", q.getCounter()) ;
    String componentId = q.getComponentId() ; 
    if(!componentId.equals("") || componentId.length() > 0) 
      oq.addLIKE("portalOwner", componentId) ;
    oq.setGroupBy("portalOwner") ;    
    String groupQuery = oq.getHibernateGroupByQuery() ;
    Session session = hservice_.openSession() ;
    if(q.getMostAccess() != "")
      groupQuery += " order by sum(o.accessCounter) desc " ;
    return session.createQuery(groupQuery).list() ;
  }
  
  public List getCachedObjects(int timeIndex) throws Exception{
    List dataList = portal_[timeIndex].getCachedObjects() ;
    portal_[timeIndex].clearCache() ;
    return  dataList ;
  }
  
  public void  saveMonitorData() throws Exception {
    List cacheObjectList = getCachedObjects(currentTimeIndex_) ;
    for(int i = 0; i < cacheObjectList.size(); i++){
      PortalLogDataImpl data = (PortalLogDataImpl)cacheObjectList.get(i) ;
      savePortalLogData(data) ;
    }
  }
  
  public void savePrevTimeSlotMonitorData(int prevTimeSlot) throws Exception {
    List cacheObjectList = getCachedObjects(prevTimeSlot) ;
    for(int i = 0; i < cacheObjectList.size(); i++){
      PortalLogDataImpl data = (PortalLogDataImpl)cacheObjectList.get(i) ;
      savePortalLogData(data) ;
    }
  }
  
  public void onExpire(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
    PortalLogDataImpl data = (PortalLogDataImpl) arg2 ;
    savePortalLogData(data) ;
    if(arg0.get(arg1) != null) 
      arg0.remove(arg1) ;
  }

  public void onRemove(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
  }

  public void onPut(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
  }

  public void onGet(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
  }

  public void onClearCache(ExoCache arg0) throws Exception {
  }
}
