/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.test;

import java.io.File;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.exoplatform.services.backup.ImportExportService;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.impl.NodeNavigationData;
import org.exoplatform.services.portal.impl.PortalConfigData;
import org.exoplatform.services.portal.model.Page;
import org.exoplatform.test.BasicTestCase;

import sun.security.krb5.internal.util.o;

import com.sun.corba.se.spi.servicecontext.UEInfoServiceContext;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Dec 2, 2005
 */
abstract class PortalServiceTestBase extends BasicTestCase {
  
  protected static ImportExportService backupService_ ;
  protected static  OrganizationService orgService_ ;
  protected static HibernateService hservice_;  
  
  protected static String demo  = "demo" ;
  protected static String Group1 = "Group1" ;
  protected static String Group2 = "Group2" ;
  protected static String username1 = "userName_1";
  protected static String username2 = "userName_2" ;
  protected static String memtype1 = "MembershipType_1" ;
  protected static String memtype2 = "MembershipType_2" ;   
    
  protected Group group1, group2, groupDefault;  
  protected MembershipType mType1,mType2, mTypeDefault ;
  protected User user1, user2 ,userDefault;     
  
  public PortalServiceTestBase(String name) {
    super(name);    
  }
  
  
  public void setUp() throws Exception {    
  }
  
  public void tearDown() throws Exception {    
  }
  
  protected void prepareOrganizationData() throws Exception{
    groupDefault = orgService_.getGroupHandler().findGroupById("/users") ;         
    if(group1 ==null) { group1 = createGroup( Group1); }    
    if(group2 ==null) { group2 = createGroup( Group2) ; }
    
    mTypeDefault = orgService_.getMembershipTypeHandler().findMembershipType("member") ;
    if(mType1 ==null) { mType1 = createMembershipType(memtype1); }    
    if(mType2 ==null) {mType2 = createMembershipType(memtype2); 
      }
            
    if(user1 ==null) {
      user1 =  createUser(username1);
      createDataUser(user1);            
    }    
    if(user2 ==null) {
      user2= createUser(username2) ;    
      createDataUser(user2) ;            
    }
    userDefault = orgService_.getUserHandler().findUserByName(demo) ;
  }            
  protected void removeOrganizationData() throws Exception {    
    if(orgService_.getGroupHandler().findGroupById("/"+Group1) !=null) {      
      orgService_.getGroupHandler().removeGroup(group1,true) ;
      group1 = null ;
      }        
    if(orgService_.getGroupHandler().findGroupById("/"+Group2) !=null) {      
      orgService_.getGroupHandler().removeGroup(group2,true) ;
      group2 = null ;
      }
        
    if(orgService_.getMembershipTypeHandler().findMembershipType(memtype1)!= null) {      
      orgService_.getMembershipTypeHandler().removeMembershipType(memtype1,true) ;
      mType1 = null ;
      }    
    if(orgService_.getMembershipTypeHandler().findMembershipType(memtype2)!= null) {      
      orgService_.getMembershipTypeHandler().removeMembershipType(memtype2,true) ;
      mType2 = null ;
      }
            
    if(orgService_.getUserHandler().findUserByName(username1) != null) {      
      orgService_.getUserHandler().removeUser(username1,true) ;
      user1 = null ;
      }    
    if(orgService_.getUserHandler().findUserByName(username2) != null) {      
      orgService_.getUserHandler().removeUser(username2,true) ;
      user2 = null ;
      }                        
  }
  protected Group createGroup(String groupName) throws Exception {    
    Group groupParent = orgService_.getGroupHandler().createGroupInstance() ;
    groupParent.setGroupName( groupName);
    groupParent.setDescription("This is description");
    orgService_.getGroupHandler().createGroup( groupParent, true);   
    return groupParent;
  }
      
  protected MembershipType createMembershipType(String name) throws Exception {
    MembershipType mt = orgService_.getMembershipTypeHandler().createMembershipTypeInstance();
    mt.setName( name) ;
    mt.setDescription("This is a test") ;
    mt.setOwner("exo") ;    
    orgService_.getMembershipTypeHandler().createMembershipType(mt, true);
    return mt;
  }
  
  protected User createUser(String userName) throws Exception {   
    User user = orgService_.getUserHandler().createUserInstance() ;
    user.setUserName(userName) ;
    user.setPassword("default") ;
    user.setFirstName("default") ;
    user.setLastName("default") ;
    user.setEmail("exo@exoportal.org") ;
    orgService_.getUserHandler().createUser(user, true);
    return user ;
  }
  
  protected User createDataUser( User u) throws Exception {
    UserProfile up = orgService_.getUserProfileHandler().findUserProfileByName( u.getUserName());
    up.getUserInfoMap().put("user.gender", "male");
    orgService_.getUserProfileHandler().saveUserProfile(up, true);    
    return u;
  }
  
  protected ZipFile getZipServiceData(String serviceName) throws Exception{
    String zipFileName = backupService_.getServiceDataDirectory() + serviceName+ ".zip" ;
    ZipFile zipFile  = new ZipFile(new File(zipFileName)) ;
    return zipFile ;    
  }
   
  protected ZipFile getZipUserData(String userName) throws Exception{
    String zipFileName = backupService_.getUserDataDirectory() + userName+ ".zip" ;
    ZipFile zipFile  = new ZipFile(new File(zipFileName)) ;
    return zipFile ;    
  }
  
  protected boolean hasEntry(ZipFile zfile, String entryName) throws Exception {
    Enumeration e =  zfile.entries() ;
    while(e.hasMoreElements()) {
      ZipEntry entry = (ZipEntry) e.nextElement() ;
      if((entry.getName().endsWith(entryName))&&(entry.getSize()>0)) {return true ;}
     }    
    return false ;
  }
  
  protected boolean hasEntry(ZipFile zfile, String[] entryNames) throws Exception {
    Enumeration e =  zfile.entries() ;
    while(e.hasMoreElements()) {
      ZipEntry entry = (ZipEntry) e.nextElement() ;
      for(int i=0;i<entryNames.length;i++) {
        if((entry.getName().endsWith(entryNames[i]))&&(entry.getSize()>0)) {return true ; }
      }      
     }    
    return false ;
  }
  
  public static class GroupComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Group g1 = (Group) o1 ;
      Group g2 = (Group) o2 ;
      if(g1.getId().equals(g2.getId()))  return 0 ;
      return -1;
    }
  }
  
  public static class MembershipTypeComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      MembershipType mType1 = (MembershipType) o1 ;
      MembershipType mType2 = (MembershipType) o2;
      if(mType1.getName().equals(mType2.getName())) return 0 ;  
      return -1;
    }    
  }  
  public static class MembershipComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Membership m1 = (Membership) o1 ;
      Membership m2 = (Membership) o2 ;      
      if ((m1.getGroupId().equals(m2.getGroupId()))
        &&(m1.getMembershipType().equals(m2.getMembershipType()))
        &&(m1.getUserName().equals(m2.getUserName()))
        ) {return 0 ;}
      return -1;
    }    
  }    
  
  public static class PortalConfigDataComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      PortalConfigData p1 = (PortalConfigData) o1 ;
      PortalConfigData p2 = (PortalConfigData) o2 ;      
      if((p1.getOwner().equals(p2.getOwner()))
          &&(p1.getId().equals(p2.getId()))
          &&(p1.getEditPermission().equals(p2.getEditPermission()))
          &&(p1.getViewPermission().equals(p2.getViewPermission()))
          &&(p1.getPortalConfig().getDecorator().equals(p2.getPortalConfig().getDecorator()))
          &&(p1.getPortalConfig().getLocale().equals(p2.getPortalConfig().getLocale()))
          &&(p1.getPortalConfig().getRenderer().equals(p2.getPortalConfig().getRenderer()))
          ) { return 0 ; }
      return -1;
    }    
  }
  
  public static class PageConfigDataComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Page page1 = (Page) o1 ;
      Page page2 = (Page) o2 ;
      
      if(
          (page1.getEditPermission().equals(page2.getEditPermission()))
          &&(page1.getName().equals(page2.getName()))
          &&(page1.getOwner().equals(page2.getOwner()))
          &&(page1.getDecorator().equals(page2.getDecorator()))
          &&(page1.getTitle().equals(page2.getTitle()))
          &&(page1.getViewPermission().equals(page2.getViewPermission()))) { return 0 ; }
      return -1;
    }    
  }
  
  public static class NavigationConfigDataComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      NodeNavigationData nnd1 = (NodeNavigationData) o1 ;
      NodeNavigationData nnd2 = (NodeNavigationData) o2 ;
      if((nnd1.getId().equals(nnd2.getId()))
          &&(nnd1.getOwner().equals(nnd2.getOwner()))
          &&(nnd1.getNodeNavigation().getNode().getName().equals(nnd2.getNodeNavigation().getNode().getName()))
          ) {return 0 ;}
      return -1;
    }    
  }    
}
