/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.test;

import java.util.List;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.community.CommunityConfigService;
import org.exoplatform.services.portal.community.CommunityNavigation;
import org.exoplatform.services.portal.community.CommunityPortal;
import org.exoplatform.test.BasicTestCase;
/**
 * Thu, May 15, 2003 @   
 * @author: Tuan Nguyen
 * @version: $Id: TestConverter.java,v 1.6 2004/07/20 12:41:09 tuan08 Exp $
 * @since: 0.0
 * @email: tuan08@yahoo.com
 */
public class TestCommunityConfigService  extends PortalServiceTestBase {
  
  private CommunityConfigService  service_ ;                 
  public TestCommunityConfigService(String name) {
    super(name);
  }

  public void setUp() throws Exception {    
    PortalContainer manager  = PortalContainer.getInstance();
    service_ = (CommunityConfigService) manager.getComponentInstanceOfType(CommunityConfigService.class) ;
    orgService_ = (OrganizationService)manager.getComponentInstanceOfType(OrganizationService.class) ;
  }
  
  public void tearDown() throws Exception {
    orgService_ = null ;
  }
  public void testCommunityService() throws Exception {
    prepareOrganizationData() ;    
    /*--------All user have a default mapped portal and mapped navigation---*/
    CommunityPortal cp = service_.findCommunityPortal("demo") ;
    assertTrue("expect community portal for /user", cp != null) ;
    assertTrue("expect portal is default portal",cp.getPortal().equals("portal")) ;
    assertTrue("expect priority is 5(default):",cp.getPriority()==5) ;
    
    cp = service_.findCommunityPortal(username1) ;
    assertTrue("expect community portal for /user", cp != null) ;
    
    cp = service_.findCommunityPortal(username2) ;
    assertTrue("expect community portal for /user", cp != null) ;
        
    List list = service_.findCommunityNavigation("demo") ;
    assertTrue("expect one shared navigations", list.size() == 1) ;
    
    list = service_.findCommunityNavigation(username1) ;
    assertTrue("expect one shared navigations", list.size() == 1) ;
    
    list = service_.findCommunityNavigation(username2) ;
    assertTrue("expect one shared navigations", list.size() == 1) ;
    
    CommunityNavigation cn = (CommunityNavigation)list.get(0) ;
    assertTrue("expect priority is:0(default) ",cn.getPriority()==0) ;
    assertTrue("expect navigation is: portal",cn.getNavigation().equals("portal")) ;
    assertTrue("expect group is :users",cn.getGroupId().equals("/users")) ;
    assertTrue("expect membership is :member",cn.getMembership().equals("member")) ;
    
    System.out.println("------------test portal community config with----------") ;
    /*---------------portal community-----------------------
     * ---------- add a portal community for user1 in group1--------*/    
    orgService_.getMembershipHandler().linkMembership(user1,group1,mType1,true) ;
    addPortalCommunity(Group1,"community",memtype1,4) ;
    
    /*-------user1's portal is mapped to 'community' because priority of portal community config 
     * -----------for user1 in group1 is higher than it in default group(/users)---------*/
    cp = service_.findCommunityPortal(username1)  ;    
    assertTrue("portal community config is found",cp!=null) ;
    assertTrue("Expect portal is 'community'", cp.getPortal().equals("community")) ;
    assertTrue("Expect groupId is:'group 1'",cp.getGroupId().equals("/"+Group1)) ;
    assertTrue("Expect membership is mType_1",cp.getMembership().equals(memtype1)) ;
    assertTrue("Expect 1 portal community is found in group1:'community'",
        service_.getCommunityPortal("/"+Group1).getPortal().equals("community")) ;
    /*----------add a portal community config for user1 in group2------*/
    orgService_.getMembershipHandler().linkMembership(user1,group2,mType2,true) ;
    addPortalCommunity(Group2,"demo",memtype2,3) ;            
    
    assertTrue("Expect 1 portal community is found in group1:'community'",
        service_.getCommunityPortal("/"+Group1).getPortal().equals("community")) ;
    assertTrue("Expect 1 portal community config is found in group2:'demo'",
        service_.getCommunityPortal("/"+Group2).getPortal().equals("demo")) ;
    /*-------priority of portal community config for user1 in group2------- 
     * -------------is higher than it in group1--------------------*/
    cp = service_.findCommunityPortal(username1) ;
    assertTrue("Expect portal config is found", cp!=null) ;
    assertTrue("Expect portal is: demo",cp.getPortal().equals("demo")) ;
    /*-----update portal config for user1 in group1---------*/
    cp = service_.getCommunityPortal("/"+Group1) ;
    cp.setPriority(2) ;
    cp.setPortal("admin") ;
    service_.addCommunityPortal(cp) ;    
    /*-------now,priority of portal community config for user1 in group1------- 
     * -------------is higher than it in group2--------------------*/
    cp = service_.findCommunityPortal(username1) ;
    assertTrue("Expect portal config is found", cp!=null) ;
    assertTrue("Expect portal is: admin",cp.getPortal().equals("admin")) ;
    assertTrue("Expect priority is:2",cp.getPriority()==2) ;
    /*------ remove portal community congfig for user1 in group1-----
     * -----expect portal community config for this user is portal config in group2---*/
    cp = service_.getCommunityPortal("/"+Group1) ;
    service_.removeCommunityPortal(cp) ;
    cp = service_.findCommunityPortal(username1) ;
    assertTrue("Now,expect portal is: demo",cp.getPortal().equals("demo")) ;
    assertTrue("Expect group is : group2",cp.getGroupId().equals("/"+Group2)) ;
    System.out.println("------------end test portal community config ----------") ;
    
    System.out.println("------------test navigation community config ----------") ;
    /*---------------add navigation config for user1 in group1---------------*/ 
    addNavigationCommunity(Group1,"community",memtype1,6) ;    
    /*--------user 1 has 2 navigation community: 1 default('portal' nav) and 'community' nav */
    list = service_.findCommunityNavigation(username1) ;    
    assertTrue("expect 2 navigation community found",list.size()==2) ;
    
    /*--------update navigation config in group1--------*/    
    cn = service_.getCommunityNavigation("/"+Group1) ;
    cn.setMembership(memtype1) ;
    cn.setPriority(8) ;
    cn.setNavigation("demo") ;
    service_.addCommunityNavigation(cn) ;
    /*---------all information is updated------*/            
    list = service_.findCommunityNavigation(username1) ;    
    assertEquals("expect 2 navigation config are found",2,list.size()) ;
    assertTrue("expect 2 navigation config is found", list.size()==2) ;
    /*---------------------------------------------------*/
    cn = (CommunityNavigation)list.get(1) ;    
    assertTrue("Expect navigation is:'demo'", cn.getNavigation().equals("demo")) ;
    assertTrue("Expect prioritt is:8",cn.getPriority()== 8) ;    
    
    /*---------------add navigation config for user1 in group2---------------*/
    orgService_.getMembershipHandler().linkMembership(user1,group2,mType2,true) ;
    addNavigationCommunity(Group2,"admin",memtype2,6) ;
    
    list = service_.findCommunityNavigation(username1) ;    
    assertTrue("expect 3 navigation community found",list.size()==3) ;
    /*--------remove a navigation config --------*/
    cn = service_.getCommunityNavigation("/"+Group1) ;
    service_.removeCommunityNavigation(cn) ;
    list = service_.findCommunityNavigation(username1) ;
    assertTrue("Expect 2 navigation community is found",list.size()==2) ;
    System.out.println("------------End test navigation community config ----------") ;
    //------------remove a membership type--------------------
    /*--------------expect all navigation community, portal community 
     * ----associate with this membership type  are remove-------------*/
    orgService_.getMembershipTypeHandler().removeMembershipType(memtype2,true) ;
    //assertTrue("expect portal community config in group2 is removed",service_.getCommunityPortal("/"+Group2)==null) ;
    /*-----now, there is only 1 navigation community of user1:(default) 'users' nav----*/
    list = service_.findCommunityNavigation(username1) ;
    assertEquals("Expect 1 nav community are found",1, list.size()) ;
    cn = (CommunityNavigation)list.get(0) ;    
    assertTrue("Expect navigation community is :portal",cn.getNavigation().equals("portal")) ;
    assertTrue("Expect group is :users",cn.getGroupId().equals("/users")) ;
    //assertTrue("expect navigation config in group2 is removed",service_.getCommunityNavigation("/"+Group2)==null) ;        
    /*-----now, portal community config is default(/users-portal)----*/
    cp = service_.findCommunityPortal(username1) ;
    assertTrue("expect portal is default portal",cp.getPortal().equals("portal")) ;    
    assertTrue("Expect group is:/user",cp.getGroupId().equals("/users")) ;
    
    //-------------------------remove a group----------------
    /*-------------add community navigation config in group2  for user1-----------*/
    orgService_.getMembershipHandler().linkMembership(user1,group2,mType1,true) ;
    addNavigationCommunity(Group2,"exo",memtype1,4) ;
    list = service_.findCommunityNavigation(username1) ;
    assertEquals("Expect 1 nav community are found",2, list.size()) ;
    /*------------add community portal config in group2 for user1-------*/
    addPortalCommunity(Group2,"community",memtype1,3) ;
    cp = service_.findCommunityPortal(username1) ;
    assertTrue("expect portal community for user1 is:'community'",cp.getPortal().equals("community")) ;
    assertTrue("expect groups for  is:'group2'",cp.getGroupId().equals("/"+Group2)) ;
    /*---------------------remove group2---------------*/
    orgService_.getGroupHandler().removeGroup(group2,true) ;
    assertTrue("expect portal community config in group2 is removed",service_.getCommunityPortal("/"+Group2)==null) ;
    /*----------now, portal community of user1 is default(portal/users)-------*/
   cp = service_.findCommunityPortal(username1) ;
   assertTrue("expect portal community is: portal",cp.getPortal().equals("portal")) ;
   assertTrue("expect group is :/users",cp.getGroupId().equals("/users")) ;
   /*-----------now, there is only 1 navigation community config for user 1------*/
   assertTrue("expect navigation community config in group2 is removed",service_.getCommunityNavigation("/"+Group2)==null) ;
   list = service_.findCommunityNavigation(username1) ;
   assertTrue("expect 1 navigation community",list.size()==1) ;
   cn = (CommunityNavigation) list.get(0) ;
   assertTrue("Expect navigation community is :portal",cn.getNavigation().equals("portal")) ;
   assertTrue("Expect group is :users",cn.getGroupId().equals("/users")) ;
   
   // ----------------------remove a user---------------------
   /*------ set portal community and navigation community of user2 maped to user1----*/
   orgService_.getMembershipHandler().linkMembership(user2,group1,mType2,true) ;
   addPortalCommunity(Group1,username1,memtype2,3) ;   
   addNavigationCommunity(Group1,username1,memtype2,7) ; 
      
   assertTrue("expect portal community is:user1",service_.findCommunityPortal(username2).getPortal().equals(username1)) ;
   list = service_.findCommunityNavigation(username2) ;
   assertTrue("expect 2 navigation community are found",list.size()==2) ;
   cn = (CommunityNavigation)list.get(1) ;
   assertTrue("expect nav community is: user1",cn.getNavigation().equals(username1)) ;
   /*-----------remove user1, expect portal community and navigation community---- 
    * --------------relate with user1 will be removed---------------*/
   orgService_.getUserHandler().removeUser(username1,true) ;
   assertTrue("expect portal community of 'User1' is removed",service_.findCommunityPortal(username1)==null) ;
   //assertTrue("expect navigation community of 'User1' is removed",service_.findCommunityNavigation(uName_1)==null) ;
   
   /*---------now, 'User2' is mapped to default portal: portal/user*/
   cp = service_.findCommunityPortal(username2) ;
   //assertTrue("expect portal community config is:portal",cp.getPortal().equals("portal")) ;
   //assertTrue("expect group is:/users",cp.getGroupId().equals("/users")) ;
   
   /*------ now, there is only 1 nav community config is mapped with User2-------------*/
   list = service_.findCommunityNavigation(username2) ;
   cn = (CommunityNavigation) list.get(0) ;
   //assertTrue("Expect navigation community is :portal",cn.getNavigation().equals("portal")) ;
   //assertTrue("Expect group is :users",cn.getGroupId().equals("/users")) ;
   /*----------------Clean database-----*/
   removeOrganizationData() ;
  }
  
  protected String getDescription() {
    return "Test Community Config Service" ;
  }
  
  private void addNavigationCommunity(String group,String portal,String membership, int priority) throws Exception {
    CommunityNavigation cn = service_.createCommunityNavigation() ;
    cn.setGroupId("/"+group) ;
    cn.setNavigation(portal) ;
    cn.setMembership(membership) ;
    cn.setPriority(priority) ;
    service_.addCommunityNavigation(cn) ;
  } 
  
  private void addPortalCommunity(String group,String portal,String membership, int priority) throws Exception {
    CommunityPortal cp = service_.createCommunityPortal() ;
    cp.setGroupId("/"+group) ;
    cp.setPortal(portal) ;
    cp.setMembership(membership) ;
    cp.setPriority(priority) ;    
    service_.addCommunityPortal(cp) ;
  }    
}