/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.test.portlet.Organization ;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.conditiongroovy.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import org.exoplatform.test.portlet.Account.*;
import org.exoplatform.test.portlet.Login.*;
/**
 * May 21, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: OrganizationPortletSuite.java,v 1.3 2004/10/22 14:23:55 tuan08 Exp $
 **/
public class GroupManagementSuite extends WebUnitSuite {
	static String DESC = "Go to Group Management page and test all the basic actions at the users tab" ;
  static String NAME = "GroupManagementSuite" ;
  final static public String EXOPLATFORM_ACCOUNT = "exoplatform" ;
    
  public GroupManagementSuite() {
    super(NAME, DESC) ;
  	init("/Site Map/Management", "Site Map/User Registration") ;
  }  
  public GroupManagementSuite(String path, String reg_path) {
    super(NAME, DESC) ;
  	init(path, reg_path) ;
  }  
  public void  init(path, reg_path) {
    goToPage(path) ;
    //delete if exoplatform accout already existed
    addWebUnit(
       new ClickURLLinkUnit("DeleteIfExistedAccount", "Delete the test existed account").
       setURL("op=delete&objectId="+EXOPLATFORM_ACCOUNT).
       addCondition(new HaveLinkWithURLCondition("op=delete&objectId="+EXOPLATFORM_ACCOUNT))
    ) ;
    goToPage(reg_path) ;    
    //create exoplatform account 
    addWebUnit(AccountUnit.createNewAccountUnit(EXOPLATFORM_ACCOUNT, EXOPLATFORM_ACCOUNT)) ;
    goToPage(path);
    //======================group tab==========================================
    addWebUnit(
      new SubmitFormUnit("SubmitSearcUserForm", "search for exoplatform user").
      setFormId("quickSearchForm").
      setField("searchField","username").
      setField("term", EXOPLATFORM_ACCOUNT).
      setField("op","search")
    ) ;
   
    addWebUnit(
      new ClickURLLinkUnit("GoToGroupsTab", "Click on the groups tab").
      setURL("op=selectNode&objectId=UIGroupNode").              
      addValidator(new ExpectTextValidator("Add Group"))
    ) ;
    //create a new group   
    addWebUnit(
      new ClickLinkWithText("ClickAddGroup", "Go to the group form").
      setTextLink("Add Group").
      addValidator(new ExpectFormIdValidator("groupForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("CreateNewGroup", "Create 'test' group").
      setFormId("groupForm").      
      setField("groupName", "newgroup").
      setField("description", "a new group").
      setField("op", "save").
      addValidator(new ExpectLinkWithTextValidator("newgroup"))
    ) ;
    //go to user list , select an user and assign the a membership 
    //to the user in the new created group
    addWebUnit(
      new ClickURLLinkUnit("GoToListUserTab", "Click on the users tab").
      setURL("op=selectNode&objectId=usernode").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIListUser&op=viewUserInfo"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickViewUserInfo", "Click the view user info icon").
      setURL("op=viewUserInfo&objectId="+ EXOPLATFORM_ACCOUNT).
      addValidator(new ExpectFormIdValidator("membershipForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SubmitNewMembership", "Assign 'member' to the user in the 'newgroup' group").
      setFormId("membershipForm").
      setField(WebUnit.ACTION, "save").
      setField("membershipName", "member").
      setField("groupId", "/newgroup").
      addValidator(new ExpectTextValidator("newgroup"))
    ) ;
    addWebUnit(
      new ClickLinkWithText("ClickBackLink", "Go back to the user list screen").
      setTextLink("back").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIListUser&op=viewUserInfo"))
   ) ;
    
    // go to the group tab
    addWebUnit(
      new ClickURLLinkUnit("GoToGroupsTab", "Click on the groups tab").
      setURL("op=selectNode&objectId=UIGroupNode").
      addValidator(new ExpectTextValidator("Add Group"))
    ) ;
    addWebUnit(
      new ClickLink("Click:newgroup", "Go to 'newgroup'").
      setTextLink("newgroup").
      addValidator(new ExpectTextValidator(EXOPLATFORM_ACCOUNT))
    ) ;         
    // add community
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCommunityConfigTab", "Click on Community Config tab'").
      setURL("&uicomponent=UIGroupDetail&op=selectNode&objectId=UICommunityNode").
      addValidator(new ExpectTextValidator("add/edit"))
    ) ;     
    addWebUnit(
      new ClickURLLinkUnit("AddEditPortalCommunity", "Click on add/edit'").
      setURL("&uicomponent=UIGroupCommunityInfo&op=addEditPortal").
      addValidator(new ExpectFormIdValidator("communityPortalForm"))      
    ) ;     
    addWebUnit(
      new SubmitFormUnit("SubmitCommunityPortalForm", "Submit community portal form").
      setFormId("communityPortalForm").
      setField(WebUnit.ACTION, "save").
      setField("portal", EXOPLATFORM_ACCOUNT).
      setField("priority", "1").
      addValidator(new ExpectTextValidator("add/edit"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("AddEditNavigationCommunity", "Click on add/edit'").
      setURL("&uicomponent=UIGroupCommunityInfo&op=addEditNav").
      addValidator(new ExpectFormIdValidator("communityNavForm"))      
    ) ;     
    addWebUnit(
      new SubmitFormUnit("SubmitCommunityNavForm", "Submit navigation portal form").
      setFormId("communityNavForm").
      setField(WebUnit.ACTION, "save").
      setField("navigation", EXOPLATFORM_ACCOUNT).
      setField("priority", "1").
      addValidator(new ExpectTextValidator("add/edit"))
    ) ;
    // remove newgroup
    addWebUnit(
      new ClickLink("ClickRemoveGroup", "Remove 'newgroup'").
      setTextLink("Remove Group").
      addValidator(new ExpectTextValidator("Add Group")).
      addValidator(new ExpectLinkWithTextValidator("newgroup", 0))
    ) ;    
  }
  
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite("admin", "exo", roles)] ;
  }
}
