/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.maven2.plugin.exo;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.HashSet;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.exoplatform.maven2.plugin.Utils;
/** 
 * Created by The eXo Platform SARL
 * Author : Roman Pedchenko
 *          lautarul@gmail.com
 * Dec 20, 2005
 */
/**
 * @goal jcr
 * @requiresDependencyResolution runtime
 * @description mvn exo:jcr -Ddeploy=param for deploy JCR standalone
 */
public class ExoJcrDeploy extends AbstractMojo {   
  /**
   * @parameter
   * @required
   */
  private String workingDir;
  /**
   * @parameter
   * @required
   */
  private String sharedResourcesDir;
  /**
   * @parameter expression="${deploy}"
   */
  private String deploy ;
  /**
   * @parameter
   */
  private String excludeProjects ;
  /**
   * @parameter expression="${project}"
   * @required
   */
  protected MavenProject project;
  /**
   * @parameter
   */
  private String deployJonasDir;  
  /**
   * @parameter
   */
  private String deployJbossDir;
  
  public void execute() throws MojoExecutionException{
    if(!"exo-jcr".equals(project.getPackaging())) printInfo() ;
    try {  
      if (deploy != null) {
        execDeploy() ;
        return;
      }
    }catch (Exception e) {
      e.printStackTrace() ;
    }
    printInfo() ;
  }   
  
  protected HashSet<String> getIgnoreProjects() {
    HashSet<String> ignoreProjects = new HashSet<String>() ;
    if(excludeProjects != null) {
      String[] pro =  excludeProjects.split(",") ;
      for(String s : pro) ignoreProjects.add(s.trim()) ;
    }
    return ignoreProjects ;
  }
  
  private void execDeploy() throws Exception {
//    File deployWar = new File(project.getBasedir().toString() + "/../../exo-jcr/target/" + project.getBuild().getFinalName() + ".war");
//    File dirWar = new File(project.getBasedir().toString() + "/target/");
//    Utils.copyFileToDirectory(deployWar, dirWar);
    if (deploy.equals("tomcat")) {
      File deployTomcatDirFile = new File(workingDir+"/exo-tomcat");
      deployTomcat(deployTomcatDirFile, getIgnoreProjects());
    } else if(deploy.equals("jonas")) {
      File deployJonasDirFile  = new File(deployJonasDir);
      deployJonas(deployJonasDirFile, getIgnoreProjects());
    } else if(deploy.equals("jboss")) {
      File earDir = new File(deployJbossDir+"/jcr.sar");
      deployJboss(earDir,getIgnoreProjects());
    } else {
      Utils.printMessage("info","The task 'exo:jcr -Ddeploy=" + deploy+ "' is invalid !\n");
      printInfo();
    }
  }
  
  protected void deployTomcat(File deployTomcatDir, HashSet<String> ignoreProjects) throws Exception  {
    File directoryJar = new File(deployTomcatDir + "/common/lib");
    File directoryWar = new File(deployTomcatDir + "/webapps");
    directoryJar.mkdirs();
    directoryWar.mkdirs();
    Utils.deployedDependency(directoryJar, null, project, ignoreProjects) ;
    Utils.deployProject(directoryJar, directoryWar, project, true, ignoreProjects) ;
    Utils.patchConfig(new File(sharedResourcesDir + "/patch-tomcat-jcr"), deployTomcatDir) ;
  }
  
  protected void deployJonas(File deployDir, HashSet<String> ignoreProjects)  throws Exception {
    File warDir = new File(deployDir + "/webapps/autoload");
    File libDir = new File(deployDir + "/lib/apps");
    warDir.mkdirs();
    libDir.mkdirs();
    Utils.deployedDependency(libDir, null, project, ignoreProjects) ;
    Utils.deployProject(libDir, warDir, project, true,  ignoreProjects);
    Utils.patchConfig(new File(sharedResourcesDir + "/patch-jonas-jcr"), deployDir);
    File[] files = warDir.listFiles();
    for(int i = 0; i < files.length; i++ ) {
      if(files[i].getName().endsWith("war")) {
        Utils.removeManifestFromJar(files[i]);
      }
    }
  }
  
  protected void deployJboss(File earDir, HashSet<String> ignoreProjects) throws Exception {
    earDir.mkdirs();
    Utils.deployedDependency(earDir, null, project, ignoreProjects) ;
    Utils.deployProject(earDir, earDir, project, true, ignoreProjects);
    Utils.createApplicationXml(earDir);
    Utils.patchConfig(new File(sharedResourcesDir + "/jcr.sar"), earDir) ;
  }
  
  private void printInfo() throws MojoExecutionException {
    String info =  
      "The 'exo:jcr' maven2 plugin is used to assemble many exo modules into an application \n" +
      "and deploy the application to a server. To run the command successfully, you need to\n" +
      "run the command in an exo-jcr module and all the dependencies modules must be built\n" +
      "before running the command mvn exo:jcr -Ddeploy=value\n\n" +
      "The valid syntax is:\n" +
      "  mvn exo:jcr -Ddeploy=tomcat\n" +
      "  This command will copy will copy the jcr module and the dependency module to the tomcat server.\n" +
      "  mvn exo:jcr -Ddeploy=jonas\n" +
      "  This command will copy will copy the jcr module and the dependency module to the jonas server.\n" +
      "  mvn exo:jcr -Ddeploy=jboss\n" +
      "  This command will copy will copy the jcr module and the dependency module to the jboss server.\n" ;
    System.out.println(info) ;
    throw new MojoExecutionException("") ;
  }
}
