/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.tools.phpbb;

import java.io.FileOutputStream;
import java.io.Reader;
import java.util.List;
import java.util.Date;
import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;
import com.ibatis.common.resources.Resources;
import com.ibatis.sqlmap.client.SqlMapClient;
import com.ibatis.sqlmap.client.SqlMapClientBuilder;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.XppDriver;
import org.exoplatform.services.communication.forum.hibernate.* ;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Nov 21, 2004
 * @version $Id$
 */
public class PHPBBForumExporter {
  private JarOutputStream out_ ;
  private XStream xstream_ ;
  private String owner_ ;
  private SqlMapClient  sqlMap_ ;
  private Date now_ = new Date() ;
  
  public PHPBBForumExporter(String owner, String sqlMapConfig, String outputFile) throws Exception {
    owner_ = owner ;
    out_ = new JarOutputStream(new FileOutputStream(outputFile)) ;
    Reader reader = Resources.getResourceAsReader(sqlMapConfig);
    sqlMap_ = SqlMapClientBuilder.buildSqlMapClient(reader, null);
    xstream_ = new XStream(new XppDriver());
    xstream_.alias("category", CategoryImpl.class);
    xstream_.alias("forum", ForumImpl.class);
    xstream_.alias("topic", TopicImpl.class);
    xstream_.alias("forum-topic", TopicBackup.class);
    xstream_.alias("post", PostImpl.class);
  }
  
  public void export() throws Exception {
    List  categories = sqlMap_.queryForList("getCategories",  "") ; ;
    for(int i = 0; i < categories.size(); i++) {
      CategoryImpl cat = (CategoryImpl) categories.get(i) ;
      cat.setOwner(owner_);
      cat.setCreatedDate(now_) ;
      cat.setModifiedBy(owner_) ;
      cat.setModifiedDate(now_);
      exportCategory(owner_, cat) ; 
    }
    out_.close() ;
  }
  
  public void exportCategory(String path ,CategoryImpl category) throws Exception {
    String categoryPath = path + "/"  + category.getId() ;
    JarEntry entry = new JarEntry(categoryPath + "/" + category.getId() +  ".category") ;
    out_.putNextEntry(entry) ;
    String xml = xstream_.toXML(category) ;
    out_.write(xml.getBytes());
    out_.closeEntry() ;
    List forums = sqlMap_.queryForList("getForums",  category) ;
    for(int i = 0; i < forums.size(); i++) {
      ForumImpl forum = (ForumImpl)  forums.get(i) ;
      forum.setOwner(owner_) ;
      forum.setCreatedDate(now_);
      forum.setModifiedBy(owner_) ;
      forum.setModifiedDate(now_);
      forum.setLastPostDate(now_) ;
      forum.setLastPostBy(owner_);
      exportForum(categoryPath ,forum) ;
    }
  }
  
  public void exportForum(String path ,ForumImpl forum) throws Exception {
    String forumPath  =  path + "/"  +forum.getId(); 
    JarEntry entry = new JarEntry(forumPath+ "/" + forum.getId() +  ".forum") ;
    out_.putNextEntry(entry) ;
    String xml = xstream_.toXML(forum) ;
    out_.write(xml.getBytes());
    out_.closeEntry() ;
    List topics = sqlMap_.queryForList("getTopics",  forum) ;
    for(int i = 0; i < topics.size();  i++) {
      TopicImpl topic = (TopicImpl) topics.get(i) ;
      exportTopic(forumPath, topic) ;
    }
  }
  
  public void exportTopic(String path ,TopicImpl topic) throws Exception {
    JarEntry entry = new JarEntry(path + "/" + topic.getId() +  ".topic") ;
    out_.putNextEntry(entry) ;
    List  posts = sqlMap_.queryForList("getPosts",  topic) ;
    PostImpl firstPost = (PostImpl) posts.get(0) ;
    PostImpl lastPost = (PostImpl) posts.get(posts.size() - 1) ;
    topic.setOwner(firstPost.getOwner()) ;
    topic.setLastPostDate(new Date(lastPost.getCreatedDate().getTime())) ;
    topic.setLastPostBy(lastPost.getOwner()) ;
    TopicBackup backup = new TopicBackup(topic, posts) ;
    String xml = xstream_.toXML(backup) ;
    out_.write(xml.getBytes());
    out_.closeEntry() ;
  }
  
  public static void main(String[] args) throws Exception {
    PHPBBForumExporter exporter  = 
      new PHPBBForumExporter("exo", "maps/SqlMapConfig.xml", "target/forum.jar") ;
    exporter.export() ;
  }
}
